/*
 * process, Distribution repositories and basic setup for Emmanuel Bigeon projects
 * Copyright (C) 2014-2018 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * 
 */
package fr.bigeon.gclc.process.io;

import java.io.IOException;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.logging.Level;
import java.util.logging.Logger;

import fr.bigeon.gclc.manager.ConsoleOutput;

/** @author Emmanuel Bigeon */
public class ConnectingConsoleOutput implements ConsoleOutput {

    private static final Logger LOGGER  = Logger
            .getLogger(ConnectingConsoleOutput.class.getName());
    /** If the undelivered message should be stored. */
    public static final int     QUEUE   = 1;
    /** If the messages should be stored in all cases. */
    public static final int     PERSIST = 1 << 1;
    private ConsoleOutput       output;
    private boolean             close   = false;

    private final boolean       persistent;
    private final boolean       queued;
    private final Deque<String> messages;
    private final int           lines;

    /** @param style the type of redirected output
     * @param lines the number of lines to store */
    public ConnectingConsoleOutput(int style, int lines) {
        super();
        this.lines = lines;
        queued = (style & QUEUE) != 0;
        persistent = (style & PERSIST) != 0;
        if (lines > 0) {
            messages = new ArrayDeque<>(lines);
        } else {
            messages = new ArrayDeque<>();
        }
    }

    private synchronized void addMessage(String text) {
        if (persistent || queued && output == null) {
            if (messages.size() == lines) {
                messages.poll();
            }
            messages.offer(text);
        }
        if (output != null) {
            try {
                output.print(text);
            } catch (final IOException e) {
                LOGGER.severe("nable to print to connecting console");
                LOGGER.log(Level.FINE, "Console error", e);
            }
        }
    }

    /* (non-Javadoc)
     * @see java.lang.AutoCloseable#close() */
    @Override
    public void close() {
        close = true;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#isClosed() */
    @Override
    public boolean isClosed() {
        return close;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#print(java.lang.String) */
    @Override
    public void print(String text) {
        addMessage(text);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#println() */
    @Override
    public void println() {
        addMessage(System.lineSeparator());
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#println(java.lang.String) */
    @Override
    public void println(String message) {
        addMessage(message + System.lineSeparator());
    }

    public synchronized void connect(ConsoleOutput output) {
        this.output = output;
        for (final String string : messages) {
            try {
                output.print(string);
            } catch (final IOException e) {
                LOGGER.severe("nable to print to connecting console");
                LOGGER.log(Level.FINE, "Console error", e);
            }
        }
        if (!persistent) {
            messages.clear();
        }
    }

    public synchronized void disconnect() {
        output = null;
    }
}
